--                                                                    --
--  procedure Test_Strings_Edit     Copyright (c)  Dmitry A. Kazakov  --
--  Test                                           Luebeck            --
--                                                 Spring, 2000       --
--                                                                    --
--  This  library  is  free software; you can redistribute it and/or  --
--  modify it under the terms of the GNU General Public  License  as  --
--  published by the Free Software Foundation; either version  2  of  --
--  the License, or (at your option) any later version. This library  --
--  is distributed in the hope that it will be useful,  but  WITHOUT  --
--  ANY   WARRANTY;   without   even   the   implied   warranty   of  --
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  --
--  General  Public  License  for  more  details.  You  should  have  --
--  received  a  copy  of  the GNU General Public License along with  --
--  this library; if not, write to  the  Free  Software  Foundation,  --
--  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.    --
--                                                                    --
--  As a special exception, if other files instantiate generics from  --
--  this unit, or you link this unit with other files to produce  an  --
--  executable, this unit does not by  itself  cause  the  resulting  --
--  executable to be covered by the GNU General Public License. This  --
--  exception  does not however invalidate any other reasons why the  --
--  executable file might be covered by the GNU Public License.       --
--____________________________________________________________________--

with Ada.Exceptions;              use Ada.Exceptions;
with Ada.Text_IO;                 use Ada.Text_IO;
with Strings_Edit;                use Strings_Edit;
with Strings_Edit.Fields;         use Strings_Edit.Fields;
with Strings_Edit.UTF8.Handling;  use Strings_Edit.UTF8.Handling;
with Strings_Edit.Roman_Edit;     use Strings_Edit.Roman_Edit;

with Strings_Edit.Integers.Superscript;
with Strings_Edit.Integers.Subscript;
with Strings_Edit.Float_Edit;
with Strings_Edit.Integer_Edit;

procedure Test_Strings_Edit is
   package Float_Edit is new Strings_Edit.Float_Edit (Float);
   package Integer_Edit is new Strings_Edit.Integer_Edit (Integer);
   use Float_Edit;
   use Integer_Edit;

   procedure Put (Text : String; Pointer : Integer) is
      Line  : String (1..80);
      Index : Integer := Line'First;
   begin
      Put (Text);
      New_Line;
      Put
      (  Line,
         Index,
         "",
         Field => Pointer - Text'First,
         Justify => Right
      );
      Put (Line, Index, "^ Pointer ");
      Integer_Edit.Put (Line, Index, Pointer);
      Put (Line (Line'First..Index - 1));
      New_Line;
   end Put;

   procedure TestJustify
             (  Value   : String;
                Field   : Natural;
                Justify : Alignment;
                Result  : String
             )  is
      Text    : String (1..68);
      Pointer : Integer;
   begin
      Pointer := 1;
      Put (Text, Pointer, Value, Field => Field, Justify => Justify);
      if Text (1..Pointer-1) /= Result then
         raise Data_Error;
      end if;
   end TestJustify;

   procedure TestFloat (Value : Float; Base : NumberBase := 10) is
      Eps     : constant :=
         Float'Compose (1.0, 1 - Float'Machine_Mantissa) * 7.0;
      Text    : String (1..68);
      Temp    : Float;
      Diff    : Float;
      Pointer : Integer;      
   begin
      Pointer := 1;
      Put (Text, Pointer, Value, Base => Base);
      Temp :=
         Float_Edit.Value
         (  Text (Text'First..Pointer - 1),
            Base    => Base,
            ToFirst => True,
            ToLast  => True
         );
      if Temp /= Value then
         Diff := abs (2.0 * (Temp - Value) / (Temp + Value));
         if Diff > Eps then
            Put
            (  "Base:" & NumberBase'Image (Base)
            &  " Diff:" & Float'Image (Diff)
            &  ">" & Float'Image (Eps)
            );
            New_Line;
            Put (Text (Text'First..Pointer - 1));
            New_Line;
            Text (Pointer..Text'Last) := (Pointer..Text'Last => ' ');
            Put (Float_Edit.Image (Temp, Base => Base));
            New_Line;
            raise Data_Error;
         end if;
      end if;
   end TestFloat;

   procedure TestRelPrecision
             (  Value  : Float;
                Small  : Integer;
                Result : String
             )  is
      Text    : String (1..68);
      Pointer : Integer;      
   begin
      Pointer := 1;
      Put (Text, Pointer, Value, RelSmall => Small);
      if Text (1..Pointer-1) /= Result then
         raise Data_Error;
      end if;
   end TestRelPrecision;

   procedure TestAbsPrecision
             (  Value  : Float;
                Small  : Integer;
                Result : String
             )  is
      Text    : String (1..68);
      Pointer : Integer;      
   begin
      Pointer := 1;
      Put (Text, Pointer, Value, AbsSmall => Small);
      if Text (1..Pointer-1) /= Result then
         raise Data_Error;
      end if;
   end TestAbsPrecision;

   procedure TestInteger (Value : Integer; Base : NumberBase := 10) is
      Text    : String (1..68);
      Temp    : Integer;
      Pointer : Integer;
   begin
      Pointer := 1;
      Put (Text, Pointer, Value, Base => Base);
      Temp :=
         Integer_Edit.Value
         (  Text (Text'First..Pointer - 1),
            Base => Base
         );
      if (  Temp /= Value
         or else
            (  Integer_Edit.Image (Temp, Base => Base)
            /= Text (Text'First..Pointer - 1)
         )  )
      then
         Put (Text (Text'First..Pointer - 1));
         New_Line;
         Text (Pointer..Text'Last) := (Pointer..Text'Last => ' ');
         Pointer := 1;
         Get (Text, Pointer, Temp, Base => Base);
         Put (Text, Pointer);
	   raise Data_Error;
      end if;
   end TestInteger;

   procedure Put_UTF8 (Text : String) is
      Line  : String (1..180);
      Index : Integer := Line'First;
   begin
      for I in Text'Range loop
         Put (Line, Index, Integer'Image (Character'Pos (Text (I))));
      end loop;
      Put_Line (Line (1..Index - 1));
   end Put_UTF8;

   procedure Test_UTF8_Superscript_Integer
             (  Value : Integer;
                Plus  : Boolean := False
             )  is
      use Integers;
      Text    : String (1..256);
      Temp    : Integer;
      Pointer : Integer;
   begin
      Pointer := 1;
      Superscript.Put (Text, Pointer, Value, PutPlus=>Plus);
      Temp := Superscript.Value (Text (Text'First..Pointer - 1));
      if (  Temp /= Value
         or else
            (  Superscript.Image (Temp, PutPlus=>Plus)
            /= Text (Text'First..Pointer - 1)
         )  )
      then
         Put_Line ("Number: " & Integers.Image (Value));
         Put_UTF8 (Text (Text'First..Pointer - 1));
         Put_Line ("Got: " & Integers.Image (Temp));
	   raise Data_Error;
      end if;
   end Test_UTF8_Superscript_Integer;

   procedure Test_UTF8_Subscript_Integer
             (  Value : Integer;
                Plus  : Boolean := False
             )  is
      use Integers;
      Text    : String (1..256);
      Temp    : Integer;
      Pointer : Integer;
   begin
      Pointer := 1;
      Subscript.Put (Text, Pointer, Value, PutPlus=>Plus);
      Temp := Subscript.Value (Text (Text'First..Pointer - 1));
      if (  Temp /= Value
         or else
            (  Subscript.Image (Temp, PutPlus=>Plus)
            /= Text (Text'First..Pointer - 1)
         )  )
      then
         Put_Line ("Number: " & Integers.Image (Value));
         Put_UTF8 (Text (Text'First..Pointer - 1));
         Put_Line ("Got: " & Integers.Image (Temp));
	   raise Data_Error;
      end if;
   end Test_UTF8_Subscript_Integer;

   procedure TestFields
             (  Destination : String;
                Value       : String;
                Field       : Natural;
                Justify     : Alignment;
                Fill        : Character;
                Expected    : String
             )  is
      procedure Put_String
                (  Destination : in out String;
                   Pointer     : in out Integer;
                   Value       : String;
                   Field       : Natural := 0;
                   Justify     : Alignment := Left;
                   Fill        : Character := ' '
                )  is
         Out_Field : constant Natural :=
                        Get_Output_Field (Destination, Pointer, Field);
         subtype Output is String (Pointer..Pointer + Out_Field - 1);
         Text  : Output renames
                    Destination (Pointer..Pointer + Out_Field - 1);
         Index : Integer := Pointer;
      begin
         Put (Text, Index, Value);
         Adjust_Output_Field
         (  Destination,
            Pointer,
            Index,
            Out_Field,
            Field,
            Justify,
            Fill
         );
      end Put_String;
      Text    : String  := Destination;
      Pointer : Integer := Text'First;
   begin
      Put_String (Text, Pointer, Value, Field, Justify, Fill);
      if Text /= Expected then
	   Raise_Exception
         (  Data_Error'Identity,
            "Got '" & Text & "', expected '" & Expected & "'"
         );
      end if;
   end TestFields;
  
   type Char_Positions is array (Positive range <>) of Integer;
   
   function By_Pos (S : Char_Positions) return String is
      R : String (S'Range);
   begin
      for I in S'Range loop
         R (I) := Character'Val (S (I));
      end loop;
      return R;
   end By_Pos;

   Text    : String (1..68);
   Pointer : Integer;
begin
   Put_Line ("Testing strings edit ...");

   for I in Character'Range loop
      if (  To_String (To_UTF8 (I)) /= (1=>I)
         or else
            To_String (To_UTF8 (String'(1=>I))) /= (1=>I)
         )
      then
	   Raise_Exception
         (  Data_Error'Identity,
            "UTF-8 <-> Latin-1 conversion error"
         );
      end if;
   end loop;

   for I in Wide_Character'Range loop
      if (  To_Wide_String (To_UTF8 (I)) /= (1=>I)
         or else
            To_Wide_String (To_UTF8 (Wide_String'(1=>I))) /= (1=>I)
         )
      then
	   Raise_Exception
         (  Data_Error'Identity,
            "UTF-8 <-> UCS-2 conversion error"
         );
      end if;
   end loop;

   declare
      use Strings_Edit.UTF8;
      Buffer     : String (1..4);
      That, This : UTF8_Code_Point;
      I1, I2     : Integer;
   begin
      for I in UTF8_Code_Point'Range loop
         I1   := Buffer'First;
         I2   := Buffer'First;
         This := I;
         Put (Buffer, I1, This);
         Get (Buffer, I2, That);
         if I1 /= I2 or This /= That then
	      Raise_Exception
            (  Use_Error'Identity,
               "UTF-8 I/O error"
            );
         end if;
      end loop;
   exception
      when Data_Error =>
         Raise_Exception
         (  Data_Error'Identity,
            "Error converting" & UTF8_Code_Point'Image (This)
         );
   end;

   declare
      use Strings_Edit.UTF8;
      use Strings_Edit.UTF8.Handling;
      procedure Show (Text : String; Code : UTF8_Code_Point) is
         UTF : String := Image (Code);
      begin
         Put (Text & ":" & UTF8_Code_Point'Image (Code) & " =");
         for I in UTF'Range loop
            Put (Integer'Image (Character'Pos (UTF (I))));
         end loop;
         New_Line;
      end Show;
   begin
      Put_Line ("Some UTF-8 code positions");
      Show ("DEGREE CELSIUS",    16#2103#);
      Show ("DEGREE FAHRENHEIT", 16#2109#);
      Show ("OHM SIGN",          16#2126#);
      Show ("KELVIN SIGN",       16#212A#);
      Show ("ANGSTROM SIGN",     16#212B#);
      Show ("SUPERSCRIPT FOUR",  16#2074#);
      Show ("SUPERSCRIPT FIVE",  16#2075#);
      Show ("SUPERSCRIPT SIX",   16#2076#);
      Show ("SUPERSCRIPT SEVEN", 16#2077#);
      Show ("SUPERSCRIPT EIGHT", 16#2078#);
      Show ("SUPERSCRIPT NINE",  16#2079#);
      Show ("SUPERSCRIPT PLUS",  16#207A#);
      Show ("SUPERSCRIPT MINUS", 16#207B#);
      Show ("SUBSCRIPT ZERO",    16#2080#);
      Show ("SUBSCRIPT ONE",     16#2081#);
      Show ("SUBSCRIPT TWO",     16#2082#);
      Show ("SUBSCRIPT THREE",   16#2083#);
      Show ("SUBSCRIPT FOUR",    16#2084#);
      Show ("SUBSCRIPT FIVE",    16#2085#);
      Show ("SUBSCRIPT SIX",     16#2086#);
      Show ("SUBSCRIPT SEVEN",   16#2087#);
      Show ("SUBSCRIPT EIGHT",   16#2088#);
      Show ("SUBSCRIPT NINE",    16#2089#);
      Show ("SUBSCRIPT PLUS",    16#208A#);
      Show ("SUBSCRIPT MINUS",   16#208B#);
      declare
         Test  : String      := "kgm/As";
         UTF   : String      := To_UTF8 (Test);
         Wide  : Wide_String := To_Wide_String (UTF);
      begin
         if To_UTF8 (Wide) /= UTF then
            Raise_Exception
            (  Data_Error'Identity,
               "Error converting to wide-string"
            );
         end if;
      end;
   end;

   Test_UTF8_Superscript_Integer (-6);
   Test_UTF8_Superscript_Integer (-987);
   Test_UTF8_Superscript_Integer (0);
   Test_UTF8_Superscript_Integer (12345);

   Test_UTF8_Superscript_Integer (-6,    True);
   Test_UTF8_Superscript_Integer (-987,  True);
   Test_UTF8_Superscript_Integer (0,     True);
   Test_UTF8_Superscript_Integer (12345, True);

   Test_UTF8_Subscript_Integer (-6);
   Test_UTF8_Subscript_Integer (-987);
   Test_UTF8_Subscript_Integer (0);
   Test_UTF8_Subscript_Integer (12345);

   Test_UTF8_Subscript_Integer (-6,    True);
   Test_UTF8_Subscript_Integer (-987,  True);
   Test_UTF8_Subscript_Integer (0,     True);
   Test_UTF8_Subscript_Integer (12345, True);

   TestJustify ("#", 10, Left,   "#         ");
   TestJustify ("#", 10, Right,  "         #");
   TestJustify ("#", 10, Center, "    #     ");

   TestJustify ("##########", 10, Left,   "##########");
   TestJustify ("##########", 10, Right,  "##########");
   TestJustify ("##########", 10, Center, "##########");

   TestRelPrecision (2.345678, 1, "2");
   TestRelPrecision (2.345678, 2, "2.3");
   TestRelPrecision (2.345678, 3, "2.35");
   TestRelPrecision (2.345678, 4, "2.346");
   TestRelPrecision (2.345678, 5, "2.3457");

   TestAbsPrecision (2.345678, 1, "0");
   TestAbsPrecision (2.345678, 0, "2");
   TestAbsPrecision (2.345678,-1, "2.3");
   TestAbsPrecision (2.345678,-2, "2.35");
   TestAbsPrecision (2.345678,-3, "2.346");
   TestAbsPrecision (2.345678,-4, "2.3457");

   for Base in NumberBase'Range loop
      TestFloat (Float'Last,  Base => Base);
      TestFloat (Float'First, Base => Base);
   end loop;

   TestFloat (1.2e37, Base => 2);
   TestFloat (1.2e37, Base => 16);
   TestFloat (1.2e37);
   TestFloat (123.45);
   TestFloat (0.00000012345);
   TestFloat (0.000000000000012345);

   for Number in Roman'Range loop
      Pointer := 1;
      Put (Text, Pointer, Number);
      if (  Roman_Edit.Value (Text (Text'First..Pointer - 1))
         /= Number
         )
      then
         Put
         (  Text (Text'First..Pointer - 1)
         &  "/="
         &  Roman_Edit.Image (Number)
         );
         New_Line;
	 raise Data_Error;
      end if;
   end loop;

   for Base in NumberBase'Range loop
      TestInteger (Integer'Last, Base => Base);
      TestInteger (Integer'First, Base => Base);
   end loop;

   TestInteger (0);
   TestInteger (-6);   
   TestInteger (12345);

   begin
      if 0.0 = Value ("e+15") then null; end if;
      raise Data_Error;
   exception
      when End_Error => null;
   end;
   begin
      if 0 = Integer'(Value ("+")) then null; end if;
      raise Data_Error;
   exception
      when Data_Error => null;
   end;
   begin
      if 0.0 = Value ("+") then null; end if;
      raise Data_Error;
   exception
      when Data_Error => null;
   end;
   begin
      if 0.0 = Value (" + .") then null; end if;
      raise Data_Error;
   exception
      when Data_Error => null;
   end;
   begin
      if 0.0 = Value ("  xx") then null; end if;
      raise Data_Error;
   exception
      when End_Error => null;
   end;
   begin
      Pointer := 0;
      Put (Text, Pointer, "");
      raise Data_Error;
   exception
      when Layout_Error => null;
   end;
   begin
      Pointer := Text'Last;
      Put (Text, Pointer, "AA");
      raise Data_Error;
   exception
      when Layout_Error => null;
   end;
   Pointer := Text'Last;
   Put (Text, Pointer, "A");
   if Pointer /= Text'Last + 1 or Text (Text'Last) /= 'A' then
      raise Data_Error;
   end if;
   Pointer := Text'Last + 1;
   Put (Text, Pointer, "");
   if Pointer /= Text'Last + 1 then
      raise Data_Error;
   end if;
   TestFields ("xxx",  "a", 0, Right,  ' ', "axx" );
   TestFields ("xxx",  "",  2, Right,  ' ', "  x" );
   TestFields ("xxx",  "s", 3, Right,  'h', "hhs" );
   TestFields ("xxxx", "t", 3, Center, ' ', " t x");
   TestFields ("xxxx", "t", 3, Left,   ' ', "t  x");
   TestFields ("xxxx", "t", 1, Left,   ' ', "txxx");
   begin
      TestFields ("xxxx", "zzzz", 3, Left,   ' ', "t  x");
      raise Data_Error;
   exception
      when Layout_Error => null;
   end;
   declare
      Text  : constant String :=
                 (  "123456789012345678901234567890"
                 &  "123456789012345678901234567890"
                 &  "123456789012345678901234567890"
                 &  "123456789012345678901234567890"
                 &  "123456789012345678901234567890"
                 &  "123456789012345678901234567890"
                 &  "E99"
                 );
      Value : Float;
   begin
      Pointer := Text'First;
      Get (Text, Pointer, Value, ToLast => True);
      if Value /= Float'Last then
         raise Constraint_Error; 
      end if;
   end;
   declare
      Text  : constant String :=
                 (  "1E"
                 &  "123456789012345678901234567890"
                 &  "123456789012345678901234567890"
                 &  "123456789012345678901234567890"
                 &  "123456789012345678901234567890"
                 &  "123456789012345678901234567890"
                 &  "123456789012345678901234567890"
                 );
      Value : Float;
   begin
      Pointer := Text'First;
      Get (Text, Pointer, Value, ToLast => True);
      if Value /= Float'Last then
         raise Constraint_Error; 
      end if;
   end;
   Put_Line ("... Done");
exception
   when Error : others =>
      Put ("Error :");
      Put_Line (Exception_Information (Error));
end Test_Strings_Edit;
